import javafx.application.Application;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.TextArea;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.image.PixelFormat;
import javafx.scene.image.PixelReader;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.stage.Stage;

public class FxImageExample3 extends Application
{
	// Create the TextArea
	private final TextArea textArea = new TextArea();
	
	public static void main(String[] args) 
	{
		Application.launch(args);
	}
	
	@Override
	public void start(Stage stage) 
	{
		// Settings for TextArea
		textArea.setMinWidth(200);
		textArea.setMinHeight(600);
		
		// Load an image in the background		
		String imageUrl = "https://docs.oracle.com/javafx/javafx/images/javafx-documentation.png";
		Image image = new Image(imageUrl);
		
		// Create the ImageView		
		ImageView imageView = new ImageView(image);

		// Read pixels from the image
		this.readPixelsInfo(image);
		
		// Create the VBox
		VBox root = new VBox();
		// Add Children to the VBox
		root.getChildren().addAll(imageView,textArea);
		
		root.setAlignment(Pos.TOP_CENTER);
		// Set the padding of the VBox
		root.setStyle("-fx-padding: 10;");
		// Set the border-style of the VBox
		root.setStyle("-fx-border-style: solid inside;");
		// Set the border-width of the VBox
		root.setStyle("-fx-border-width: 2;");
		// Set the border-insets of the VBox
		root.setStyle("-fx-border-insets: 5;");
		// Set the border-radius of the VBox
		root.setStyle("-fx-border-radius: 5;");
		// Set the border-color of the VBox
		root.setStyle("-fx-border-color: blue;");		
		// Set the size of the VBox
		root.setPrefSize(200, 600);

		// Create the Scene
		Scene scene = new Scene(root);
		// Add the scene to the Stage
		stage.setScene(scene);
		// Set the title of the Stage
		stage.setTitle("Reading Pixels from an Image");
		// Display the Stage
		stage.show();		
	}
	
	@SuppressWarnings("rawtypes")
	private void readPixelsInfo(Image image) 
	{
		// Obtain the pixel reader from the image
		PixelReader pixelReader = image.getPixelReader();
	
		if (pixelReader == null) 
		{
			System.out.println("Connot read pixels from the image");
			return;
		}
		
		// Get Properties from the Image
		int width = (int)image.getWidth();
		int height = (int)image.getHeight();
		double progress = image.getProgress();
		double requWidth = image.getRequestedWidth();
		double requHeight = image.getRequestedHeight();
		
		PixelFormat format = pixelReader.getPixelFormat();
		PixelFormat.Type formatType = format.getType();

		// Write out the Properties to the TextArea
		this.writeInfoMessage("Image Width: " + width);
		this.writeInfoMessage("Image Height: " + height);
		this.writeInfoMessage("Progress: " + progress);
		this.writeInfoMessage("Requested Image Width: " + requWidth);
		this.writeInfoMessage("Requested Image Height: " + requHeight);
		this.writeInfoMessage("Pixel format type: " + formatType);
		
		
		// Read all pixels	
		for(int y = 0; y < height; y++) 
		{
			for(int x = 0; x < width; x++) 
			{
				Color color = pixelReader.getColor(x, y);
				this.writeInfoMessage("Color at (" + x + ", " + y + ") = " + color);
			}
		}
	}	
	
	// Method for Logging
	private void writeInfoMessage(String msg)
	{
		this.textArea.appendText(msg + "\n");
	}
	
}