import javafx.application.Application;
import javafx.scene.Scene;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.image.PixelReader;
import javafx.scene.image.PixelWriter;
import javafx.scene.image.WritableImage;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.text.Text;
import javafx.stage.Stage;

public class FxImageExample4 extends Application
{
	public static void main(String[] args) 
	{
		Application.launch(args);
	}
	
	@Override
	public void start(Stage stage) 
	{
		// Load an image in the background
		String imageUrl = "https://docs.oracle.com/javafx/javafx/images/javafx-documentation.png";
		Image image = new Image(imageUrl);

		// Set the Width and Height of the Image
		int width = (int)image.getWidth();
		int height = (int)image.getHeight();

		// Create three WritableImage instances
		// One Image will be a darker, one brighter, and one semi-transparent
		WritableImage darkerImage = new WritableImage(width, height);
		WritableImage brighterImage = new WritableImage(width, height);
		WritableImage semiTransparentImage = new WritableImage(width, height);		
		
		// Copy source pixels to the destinations
		this.createImages(image, darkerImage, brighterImage, semiTransparentImage,width,height);
		
		// Create the ImageViews
		ImageView imageView = new ImageView(image);
		ImageView darkerView = new ImageView(darkerImage);
		ImageView brighterView = new ImageView(brighterImage);
		ImageView semiTransparentView = new ImageView(semiTransparentImage);
		
		// Create the VBox for the Original Image
		VBox originalViewBox = new VBox();
		// Add ImageView to the VBox
		originalViewBox.getChildren().addAll(imageView, new Text("Original"));
		
		// Create the VBox for the Darker Image
		VBox darkerViewBox = new VBox();
		// Add ImageView to the VBox
		darkerViewBox.getChildren().addAll(darkerView, new Text("Darker"));
		
		// Create the VBox for the Brighter Image
		VBox brighterViewBox = new VBox();
		// Add ImageView to the VBox
		brighterViewBox.getChildren().addAll(brighterView, new Text("Brighter"));

		// Create the VBox for the Semi-Transparent Image
		VBox transparentViewBox = new VBox();		
		// Add ImageView to the VBox
		transparentViewBox.getChildren().addAll(semiTransparentView, new Text("Semi-Transparent"));
		
		// Create the HBox
		HBox root = new HBox(10);
		// Add VBoxes to the HBox
		root.getChildren().addAll(originalViewBox,darkerViewBox,brighterViewBox,transparentViewBox);
		
		// Set the padding of the HBox
		root.setStyle("-fx-padding: 10;");
		// Set the border-style of the HBox
		root.setStyle("-fx-border-style: solid inside;");
		// Set the border-width of the HBox
		root.setStyle("-fx-border-width: 2;");
		// Set the border-insets of the HBox
		root.setStyle("-fx-border-insets: 5;");
		// Set the border-radius of the HBox
		root.setStyle("-fx-border-radius: 5;");
		// Set the border-color of the HBox
		root.setStyle("-fx-border-color: blue;");		
		// Set the size of the HBox
		root.setPrefSize(400, 200);
		
		// Create the Scene
		Scene scene = new Scene(root);
		// Add the scene to the Stage
		stage.setScene(scene);
		// Set the title of the Stage
		stage.setTitle("Writing Pixels to an Image");
		// Display the Stage
		stage.show();		
	}
	
	private void createImages(Image image, WritableImage darkerImage,WritableImage brighterImage,
			WritableImage semiTransparentImage,	int width, int height) 
	{
		// Obtain the pixel reader from the image
		PixelReader pixelReader = image.getPixelReader();
		PixelWriter darkerWriter = darkerImage.getPixelWriter();
		PixelWriter brighterWriter = brighterImage.getPixelWriter();
		PixelWriter semiTransparentWriter = semiTransparentImage.getPixelWriter();

		// Read one pixel at a time from the source and
		// write it to the destinations
		for(int y = 0; y < height; y++) 
		{
			for(int x = 0; x < width; x++) 
			{
				Color color = pixelReader.getColor(x, y);
				// Write a darker pixel to the new image
				darkerWriter.setColor(x, y, color.darker());
				// Write a brighter pixel to the new image
				brighterWriter.setColor(x, y, color.brighter());	
				// Write a semi-transparent pixel to the new image
				semiTransparentWriter.setColor(x, y, Color.color(color.getRed(), 
						color.getGreen(),color.getBlue(), 0.50));
			}
		}
	}				
}
